﻿using ComputerShopContracts.BindingModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using ComputerShopDataModels.Enums;
using ComputerShopContracts.ViewModels;
using ComputerShopDataModels.Models;

namespace ComputerShopDatabaseImplement.Models
{
    internal class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        [ForeignKey("OrderId")]
        private Dictionary<int, (IComponentModel, int)>? _orderComponents = null;

        public Dictionary<int, (IComponentModel, int)> OrderComponents
		{
            get
            {
                if (_orderComponents == null)
                {
					_orderComponents = Components
                    .ToDictionary(recPC => recPC.ComponentId, recPC =>
                    (recPC.Component as IComponentModel, recPC.Count));
                }
                return _orderComponents;
            }
        }

		[ForeignKey("OrderId")]
		public virtual List<OrderComponent> Components { get; set; } = new();
		[ForeignKey("OrderId")]
		public virtual List<Supply>? Supplies { get; set; } = new();

		[Required]
        public int ClientId { get; set; }
		public virtual Client? Client { get; set; }
		public static Order? Create(ComputerShopDatabase context, OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            var order_comp = model.OrderComponents.Select(x => new
                OrderComponent
            {
                Component = context.Components.First(y => y.Id == x.Key),
                Count = x.Value.Item2
            }).ToList();


            return new Order()
            {
                Id = model.Id,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId,
				Components = order_comp
            };
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
			OrderComponents = OrderComponents,
            ClientId = ClientId
        };
    }
}
